@@ -25,12 +25,14 @@
 #endif
 
 #include <string.h>
+#include <stdio.h>
 
 #include <glib.h>
 #include <gio/gio.h>
 
 #include "yelp-uri.h"
 #include "yelp-debug.h"
+#include "yelp-settings.h"
 
 static void           yelp_uri_class_init        (YelpUriClass   *klass);
 static void           yelp_uri_init              (YelpUri        *uri);
@@ -50,6 +52,7 @@ static void           resolve_data_dirs          (YelpUri        *uri,
                                                   gboolean        langfirst);
 static void           resolve_ghelp_uri          (YelpUri        *uri);
 static void           resolve_help_uri           (YelpUri        *uri);
+static void           resolve_help_list_uri      (YelpUri        *uri);
 static void           resolve_man_uri            (YelpUri        *uri);
 static void           resolve_info_uri           (YelpUri        *uri);
 static void           resolve_xref_uri           (YelpUri        *uri);
@@ -113,6 +116,9 @@ static const gchar *infosuffix[] = {
     NULL
 };
 
+static const gchar default_info_path[] =
+    "/usr/info:/usr/share/info:/usr/local/info:/usr/local/share/info";
+
 /******************************************************************************/
 
 static void
@@ -263,6 +269,9 @@ resolve_async (YelpUri *uri)
     else if (g_str_has_prefix (priv->res_arg, "help:")) {
         resolve_help_uri (uri);
     }
+    else if (g_str_has_prefix (priv->res_arg, "help-list:")) {
+        resolve_help_list_uri (uri);
+    }
     else if (g_str_has_prefix (priv->res_arg, "file:")) {
         resolve_file_uri (uri);
     }
@@ -295,7 +304,7 @@ resolve_async (YelpUri *uri)
         case YELP_URI_DOCUMENT_TYPE_XHTML:
             resolve_file_path (uri);
             break;
-        case YELP_URI_DOCUMENT_TYPE_TOC:
+        case YELP_URI_DOCUMENT_TYPE_HELP_LIST:
             /* FIXME: what do we do? */
             break;
         case YELP_URI_DOCUMENT_TYPE_SEARCH:
@@ -551,7 +560,7 @@ resolve_data_dirs (YelpUri      *ret,
     gchar *filename = NULL;
     gchar **searchpath = NULL;
     gint searchi, searchmax;
-    gint datadir_i, subdir_i, lang_i;
+    gint datadir_i, lang_i;
 
     datadirs = g_new0 (gchar *, g_strv_length ((gchar **) sdatadirs) + 2);
     datadirs[0] = (gchar *) g_get_user_data_dir ();
@@ -794,6 +803,62 @@ resolve_help_uri (YelpUri *uri)
     return;
 }
 
+static void
+resolve_help_list_uri (YelpUri *uri)
+{
+    YelpUriPrivate *priv = GET_PRIV (uri);
+    priv->docuri = g_strdup ("help-list:");
+    priv->fulluri = g_strdup (priv->res_arg);
+    priv->page_id = g_strdup ("index");
+    priv->tmptype = YELP_URI_DOCUMENT_TYPE_HELP_LIST;
+}
+
+/*
+  Resolve a manual file's path using 'man -w'. section may be NULL,
+  otherwise should be the section of the manual (ie should have dealt
+  with empty strings before calling this!) Returns NULL if the file
+  can't be found.
+*/
+static gchar*
+find_man_path (gchar* name, gchar* section)
+{
+    gchar* argv[] = { "man", "-w", NULL, NULL, NULL };
+    gchar *stdout = NULL;
+    gint status;
+    gchar **lines;
+    GError *error = NULL;
+
+    /* Syntax for man is "man -w <section> <name>", possibly omitting
+       section */
+    if (section) {
+        argv[2] = section;
+        argv[3] = name;
+    } else {
+        argv[2] = name;
+    }
+
+    if (!g_spawn_sync (NULL, argv, NULL,
+                       G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+                       NULL, NULL,
+                       &stdout, NULL, &status, &error)) {
+        g_warning ("Couldn't find path for %s(%s). Error: %s",
+                   name, section, error->message);
+        g_error_free (error);
+    }
+
+    if (status == 0) {
+        lines = g_strsplit (stdout, "\n", 2);
+        g_free (stdout);
+        stdout = g_strdup (lines[0]);
+
+        g_strfreev (lines);
+        return stdout;
+    } else {
+        g_free (stdout);
+        return NULL;
+    }
+}
+
 static void
 resolve_man_uri (YelpUri *uri)
 {
@@ -803,145 +868,137 @@ resolve_man_uri (YelpUri *uri)
      * man:name.section
      * man:name
      */
-    static gchar **manpath = NULL;
-    const gchar * const * langs = g_get_language_names ();
-    gchar *newarg = NULL;
-    gchar *section = NULL;
-    gchar *name = NULL;
-    gchar *fullpath = NULL;
 
-    /* not to be freed */
-    gchar *realsection;
-    gchar *colon, *hash;
-    gchar *lbrace = NULL;
-    gchar *rbrace = NULL;
-    gint i, j, k;
+    /* Search via regular expressions for name, name(section) and
+     * name.section (assuming that name doesn't contain forward
+     * slashes or other nasties)
+     *
+     * If these don't match, assume that we were given a filename
+     * (absolute iff it starts with a /).
+     */
+    static GRegex* man_not_path = NULL;
+    GError *error = NULL;
+    GMatchInfo *match_info = NULL;
+    gchar *name, *section, *hash;
+    gchar *path;
 
-    if (g_str_has_prefix (priv->res_arg, "man:/")) {
+    if (!man_not_path) {
+        /* Match group 1 should contain the name; then one of groups 3
+         * and 4 will contain the section if there was one. Group 6
+         * will contain any hash fragment. */
+        man_not_path = g_regex_new ("man:([^ /.()#]+)"
+                                    "(\\(([0-9A-Za-z]+)\\)|\\.([0-9A-Za-z]+)|)"
+                                    "(#([^/ ()]+))?",
+                                    0, 0, &error);
+        if (!man_not_path) {
+            g_error ("Error with regex in man uri: %s\n",
+                     error->message);
+        }
+    }
+
+    if (!g_regex_match (man_not_path, priv->res_arg,
+                        0, &match_info)) {
+        /* The regexp didn't match, so treat as a file name. */
         gchar *newuri;
         priv->tmptype = YELP_URI_DOCUMENT_TYPE_MAN;
         newuri = g_strdup_printf ("file:%s", priv->res_arg + 4);
         g_free (priv->res_arg);
         priv->res_arg = newuri;
         resolve_file_uri (uri);
-        return;
     }
-
-    if (!manpath) {
-        /* Initialize manpath only once */
-        const gchar *env = g_getenv ("MANPATH");
-        if (!env || env[0] == '\0')
-            env = "/usr/share/man:/usr/man:/usr/local/share/man:/usr/local/man";
-        manpath = g_strsplit (env, ":", 0);
-    }
-
-    colon = strchr (priv->res_arg, ':');
-    if (colon)
-        colon++;
-    else
-        colon = (gchar *) priv->res_arg;
-
-    hash = strchr (colon, '#');
-    if (hash)
-        newarg = g_strndup (colon, hash - colon);
-    else
-        newarg = g_strdup (colon);
-
-    lbrace = strrchr (newarg, '(');
-    if (lbrace) {
-        rbrace = strrchr (lbrace, ')');
-        if (rbrace) {
-            section = g_strndup (lbrace + 1, rbrace - lbrace - 1);
-            name = g_strndup (newarg, lbrace - newarg);
-        } else {
-            section = NULL;
-            name = strdup (newarg);
+    else {
+        /* The regexp matched, so we've got a name/section pair that
+         * needs resolving. */
+        name = g_match_info_fetch (match_info, 1);
+        section = g_match_info_fetch (match_info, 3);
+        hash = g_match_info_fetch (match_info, 6);
+        if (!name) {
+            g_error ("Error matching strings in man uri '%s'",
+                     priv->res_arg);
         }
-    } else {
-        lbrace = strrchr (newarg, '.');
-        if (lbrace) {
-            section = strdup (lbrace + 1);
-            name = g_strndup (newarg, lbrace - newarg);
-        } else {
-            section = NULL;
-            name = strdup (newarg);
+        if ((!section) || (section[0] == '\0')) {
+            section = g_match_info_fetch (match_info, 4);
         }
-    }
+        if (section && section[0] == '\0') section = NULL;
 
-    for (i = 0; langs[i]; i++) {
-        for (j = 0; manpath[j]; j++) {
-            gchar *langdir;
-            if (g_str_equal (langs[i], "C"))
-                langdir = g_strdup (manpath[j]);
-            else
-                langdir = g_strconcat (manpath[j], "/", langs[i], NULL);
-            if (!g_file_test (langdir, G_FILE_TEST_IS_DIR)) {
-                g_free (langdir);
-                continue;
-            }
+        path = find_man_path (name, section);
 
-            for (k = 0; section ? k == 0 : mancats[k] != NULL; k++) {
-                if (section)
-                    realsection = section;
-                else
-                    realsection = (gchar *) mancats[k];
-
-                fullpath = g_strconcat (langdir, "/man", realsection,
-                                        "/", name, ".", realsection,
-                                        NULL);
-                if (g_file_test (fullpath, G_FILE_TEST_IS_REGULAR))
-                    goto gotit;
-                g_free (fullpath);
-
-                fullpath = g_strconcat (langdir, "/man", realsection,
-                                        "/", name, ".", realsection, ".gz",
-                                        NULL);
-                if (g_file_test (fullpath, G_FILE_TEST_IS_REGULAR))
-                    goto gotit;
-                g_free (fullpath);
-
-                fullpath = g_strconcat (langdir, "/man", realsection,
-                                        "/", name, ".", realsection, ".bz2",
-                                        NULL);
-                if (g_file_test (fullpath, G_FILE_TEST_IS_REGULAR))
-                    goto gotit;
-                g_free (fullpath);
-
-                fullpath = g_strconcat (langdir, "/man", realsection,
-                                        "/", name, ".", realsection, ".lzma",
-                                        NULL);
-                if (g_file_test (fullpath, G_FILE_TEST_IS_REGULAR))
-                    goto gotit;
-                g_free (fullpath);
-
-                fullpath = NULL;
-            gotit:
-                if (fullpath)
-                    break;
-            }
-            g_free (langdir);
-            if (fullpath)
-                break;
+        if (!path) {
+            priv->tmptype = YELP_URI_DOCUMENT_TYPE_NOT_FOUND;
+            return;
         }
-        if (fullpath)
-            break;
-    }
-
-    if (fullpath) {
         priv->tmptype = YELP_URI_DOCUMENT_TYPE_MAN;
-        priv->gfile = g_file_new_for_path (fullpath);
-        priv->docuri = g_strconcat ("man:",  name, ".", realsection, NULL);
+        priv->gfile = g_file_new_for_path (path);
+        priv->docuri = g_strconcat ("man:",  name, ".", section, NULL);
         priv->fulluri = g_strdup (priv->docuri);
         resolve_gfile (uri, NULL);
-        g_free (fullpath);
-    } else {
-        priv->tmptype = YELP_URI_DOCUMENT_TYPE_NOT_FOUND;
+
+        if (hash && hash[0] != '\0')
+            resolve_page_and_frag (uri, hash + 1);
+
+        g_free (path);
+        g_match_info_free (match_info);
     }
+}
 
-    if (hash)
-        resolve_page_and_frag (uri, hash + 1);
+/*
+  Return 1 if ch is a number from 0 to 9 or a letter a-f or A-F and 0
+  otherwise. This is sort of not utf8-safe, but since we are only
+  looking for 7-bit things, it doesn't matter.
+ */
+static int
+is_hex (gchar ch)
+{
+    if (((48 <= ch) && (ch <= 57)) ||
+        ((65 <= ch) && (ch <= 70)) ||
+        ((97 <= ch) && (ch <= 102)))
+        return 1;
+    return 0;
+}
 
-    g_free (newarg);
+/*
+  Return a newly allocated string, where %ab for a,b in [0, f] is
+  replaced by the character it represents.
+ */
+static gchar*
+decode_url (const gchar *url)
+{
+    if (!url) return NULL;
+
+    unsigned int len = strlen (url) + 1;
+    int hex;
+    gchar *ret = g_new (gchar, len);
+    const gchar *ptr = url, *end = url+len;
+    gchar *retptr = ret, *tmp;
+
+    while (ptr < end) {
+        if (*ptr == '%' && is_hex(*(ptr + 1)) && is_hex(*(ptr + 2))) {
+            *retptr = *(ptr+1);
+            *(retptr+1) = *(ptr+2);
+            *(retptr+2) = '\0';
+
+            sscanf (retptr, "%x", &hex);
+
+            if (hex < 0 || hex > 127) {
+                g_warning ("Skipping non-7-bit character.");
+                ptr++;
+                continue;
+            }
+            *retptr = (gchar)hex;
+
+            retptr++;
+            ptr += 3;
+        }
+        else {
+            tmp = g_utf8_next_char(ptr);
+            memcpy (retptr, ptr, (tmp-ptr));
+            retptr += tmp-ptr;
+            ptr = tmp;
+        }
+    }
+    *retptr = '\0';
+
+    return ret;
 }
 
 static void
@@ -955,7 +1012,6 @@ resolve_info_uri (YelpUri *uri)
      * info:(name)
      */
     static gchar **infopath = NULL;
-    const gchar * const * langs = g_get_language_names ();
     gchar *name = NULL;
     gchar *sect = NULL;
     gchar *fullpath = NULL;
@@ -975,10 +1031,24 @@ resolve_info_uri (YelpUri *uri)
 
     if (!infopath) {
         /* Initialize infopath only once */
+
+        /* Use the same logic as the info program. If INFOPATH is not
+           specified, use the default. If it is specified, just use it
+           unless it ends with a colon, in which case we add the
+           default as a suffix.
+        */
         const gchar *env = g_getenv ("INFOPATH");
+        gchar *paths;
         if (!env || env[0] == '\0')
-            env = "/usr/info:/usr/share/info:/usr/local/info:/usr/local/share/info";
-        infopath = g_strsplit (env, ":", 0);
+            paths = g_strdup (default_info_path);
+        else if (env[strlen (env)-1] == ':')
+            paths = g_strconcat (env, default_info_path, NULL);
+        else
+            paths = g_strdup (env);
+
+        infopath = g_strsplit (paths, ":", 0);
+
+        g_free (paths);
     }
 
     colon = strchr (priv->res_arg, ':');
@@ -1080,6 +1150,18 @@ resolve_xref_uri (YelpUri *uri)
         }
     }
 
+    if (priv->page_id &&
+        g_str_has_prefix (priv->docuri, "info:")) {
+        /*
+          Special characters get url-encoded when they get clicked on
+          as links. Info files, at least, don't want that so decode
+          the url again here.
+         */
+        gchar* tmp = priv->page_id;
+        priv->page_id = decode_url (priv->page_id);
+        g_free (tmp);
+    }
+
     if (g_str_has_prefix (priv->docuri, "ghelp:"))
         priv->fulluri = g_strconcat (priv->docuri,
                                      priv->page_id ? "?" : "",
